// #############################################################################
// # lib_goborotate.h - Gobo wheel library for 4 coil unipolar stepper motors, #
// #                    controled to make half steps                           # 
// #############################################################################
// # Version: 1.0 - Compiler: AVR-GCC 4.3.3 (Linux)                            #
// #  (c) 2009 by Malte Pöggel - www.MALTEPOEGGEL.de - malte@maltepoeggel.de   #
// #############################################################################
// #  This program is free software; you can redistribute it and/or modify it  #
// #   under the terms of the GNU General Public License as published by the   #
// #        Free Software Foundation; either version 3 of the License,         #
// #                  or (at your option) any later version.                   #
// #                                                                           #
// #      This program is distributed in the hope that it will be useful,      #
// #      but WITHOUT ANY WARRANTY; without even the implied warranty of       #
// #           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.            #
// #           See the GNU General Public License for more details.            #
// #                                                                           #
// #  You should have received a copy of the GNU General Public License along  #
// #      with this program; if not, see <http://www.gnu.org/licenses/>.       #
// #############################################################################

#ifndef GOBOROTATE_H
  #define GOBOROTATE_H

  #include <avr/io.h>
  #include <stdint.h>
  #include <avr/interrupt.h>
  #include <util/delay.h>

  // How many steps for 360 degrees
  #define MOT_STEPS 400

  // How many steps to next gobo
  #define GOBO_HALFSTEPS 25

  // Port macros
  #define DDR_COIL1 DDRA
  #define PRT_COIL1 PORTA
  #define PIN_COIL1 PA0

  #define DDR_COIL2 DDRA
  #define PRT_COIL2 PORTA
  #define PIN_COIL2 PA3

  #define DDR_COIL3 DDRA
  #define PRT_COIL3 PORTA
  #define PIN_COIL3 PA1

  #define DDR_COIL4 DDRA
  #define PRT_COIL4 PORTA
  #define PIN_COIL4 PA2

  #define DDR_SENSOR DDRB
  #define PRT_SENSOR PORTB
  #define PIN_SENSOR PB5
  #define PR_SENSOR  PINB

  #define CORR_DIR   0
  #define CORR_VAL   4

  // External functions
  extern void Init_Gobo( void ); 		// Initialize ports and zero position
  extern void SetGobo( uint8_t gobo ); 		// Set a new gobo
  extern void CheckGobo( void ); 		// Should be called periodically

#endif
