// #############################################################################
// # lib_motdriver.c - Software 2CH PWM implementation with Timer0             #
// #                   to control two DC motors via L293 bridge                #
// #############################################################################
// # Version: 1.0 - Compiler: AVR-GCC 4.3.3 (Linux)                            #
// #  (c) 2009 by Malte Pöggel - www.MALTEPOEGGEL.de - malte@maltepoeggel.de   #
// #############################################################################
// #  This program is free software; you can redistribute it and/or modify it  #
// #   under the terms of the GNU General Public License as published by the   #
// #        Free Software Foundation; either version 3 of the License,         #
// #                  or (at your option) any later version.                   #
// #                                                                           #
// #      This program is distributed in the hope that it will be useful,      #
// #      but WITHOUT ANY WARRANTY; without even the implied warranty of       #
// #           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.            #
// #           See the GNU General Public License for more details.            #
// #                                                                           #
// #  You should have received a copy of the GNU General Public License along  #
// #      with this program; if not, see <http://www.gnu.org/licenses/>.       #
// #############################################################################

#include "lib_motdriver.h"

// Compare counter
uint8_t PwmCompare=0;

// Initialize timer and interrupt
extern void Init_Motor( void )
 {
  DDR_PWMCH1  |= (1<<PIN_PWMCH1);
  DDR_PWMCH2  |= (1<<PIN_PWMCH2);
  DDR_DIRCH1R |= (1<<PIN_DIRCH1R);
  DDR_DIRCH1L |= (1<<PIN_DIRCH1L);
  DDR_DIRCH2R |= (1<<PIN_DIRCH2R);
  DDR_DIRCH2L |= (1<<PIN_DIRCH2L);
  TCCR0        = (1<<CS00);			// Set clock to sysclock
  TIMSK       |= (1<<TOIE0);			// Enable timer interrupt
 }


// Motor direction
extern inline void Motor1direction( uint8_t d )
 {
  switch(d) 
   {
    case 0:
     PRT_DIRCH1R &= ~(1<<PIN_DIRCH1R);
     PRT_DIRCH1L |=  (1<<PIN_DIRCH1L);
     break;
    case 1:
     PRT_DIRCH1R |=  (1<<PIN_DIRCH1R);
     PRT_DIRCH1L &= ~(1<<PIN_DIRCH1L);
     break;
    case 2:
     PRT_DIRCH1R &= ~(1<<PIN_DIRCH1R);
     PRT_DIRCH1L &= ~(1<<PIN_DIRCH1L);
     break;
   }
 }


extern inline void Motor2direction( uint8_t d )
 {
  switch(d) 
   {
    case 0:
     PRT_DIRCH2R &= ~(1<<PIN_DIRCH2R);
     PRT_DIRCH2L |=  (1<<PIN_DIRCH2L);
     break;
    case 1:
     PRT_DIRCH2R |=  (1<<PIN_DIRCH2R);
     PRT_DIRCH2L &= ~(1<<PIN_DIRCH2L);
     break;
    case 2:
     PRT_DIRCH2R &= ~(1<<PIN_DIRCH2R);
     PRT_DIRCH2L &= ~(1<<PIN_DIRCH2L);
     break;
   }
 }


// Interrupt vector
ISR (TIMER0_OVF_vect)
 {
  uint8_t PwmBuf=PwmCompare;
  if(PwmBuf>=PwmVal[0]) PRT_PWMCH1 &= ~(1<<PIN_PWMCH1); else PRT_PWMCH1 |= (1<<PIN_PWMCH1);
  if(PwmBuf>=PwmVal[1]) PRT_PWMCH2 &= ~(1<<PIN_PWMCH2); else PRT_PWMCH2 |= (1<<PIN_PWMCH2);
  PwmBuf++;
  PwmCompare=PwmBuf;
 }

