// #############################################################################
// # lib_motdriver.h - Software 2CH PWM implementation with Timer0             #
// #                   to control two DC motors via L293 bridge                #
// #############################################################################
// # Version: 1.0 - Compiler: AVR-GCC 4.3.3 (Linux)                            #
// #  (c) 2009 by Malte Pöggel - www.MALTEPOEGGEL.de - malte@maltepoeggel.de   #
// #############################################################################
// #  This program is free software; you can redistribute it and/or modify it  #
// #   under the terms of the GNU General Public License as published by the   #
// #        Free Software Foundation; either version 3 of the License,         #
// #                  or (at your option) any later version.                   #
// #                                                                           #
// #      This program is distributed in the hope that it will be useful,      #
// #      but WITHOUT ANY WARRANTY; without even the implied warranty of       #
// #           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.            #
// #           See the GNU General Public License for more details.            #
// #                                                                           #
// #  You should have received a copy of the GNU General Public License along  #
// #      with this program; if not, see <http://www.gnu.org/licenses/>.       #
// #############################################################################

#ifndef SOFTPWM_H
  #define SOFTPWM_H

  #include <avr/io.h>
  #include <stdint.h>
  #include <avr/interrupt.h>

  // Port macros
  #define DDR_PWMCH1 DDRB
  #define PRT_PWMCH1 PORTB
  #define PIN_PWMCH1 PB0

  #define DDR_DIRCH1R DDRA
  #define PRT_DIRCH1R PORTA
  #define PIN_DIRCH1R PA4

  #define DDR_DIRCH1L DDRA
  #define PRT_DIRCH1L PORTA
  #define PIN_DIRCH1L PA5

  #define DDR_PWMCH2 DDRB
  #define PRT_PWMCH2 PORTB
  #define PIN_PWMCH2 PB1

  #define DDR_DIRCH2R DDRA
  #define PRT_DIRCH2R PORTA
  #define PIN_DIRCH2R PA6

  #define DDR_DIRCH2L DDRA
  #define PRT_DIRCH2L PORTA
  #define PIN_DIRCH2L PA7

  // PWM Channel values
  volatile uint8_t PwmVal[2];	

  // Initialize timer and interrupt
  extern void Init_Motor( void );

  // Motor directions
  extern inline void Motor1direction( uint8_t d );
  extern inline void Motor2direction( uint8_t d );

#endif

